//+------------------------------------------------------------------+
//|                                     PresidentialTerms-RepDem.mq5 |
//|                               Copyright 2014-2021, EarnForex.com |
//|                                       https://www.earnforex.com/ |
//+------------------------------------------------------------------+
#property copyright "Copyright 2014-2021, EarnForex.com"
#property link      "https://www.earnforex.com/guides/presidential-cycles-in-forex/"
#property version   "1.02"

// Compares profit for buying a currency pair on Republican and Democratic terms.

enum parties
{
   Democrat,
   Republican,
   None
};

struct terms
{
   datetime date;
   parties party;
};

terms PresidentialTerms[7];

//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
{
   // Real date of 1969.01.20 cannot be set for Nixon because dates start from 1970 in MetaTrader.
   // Nixon/Ford
   PresidentialTerms[0].date = D'1970.01.01';
   PresidentialTerms[0].party = Republican;
   // Carter
   PresidentialTerms[1].date = D'1977.01.20';
   PresidentialTerms[1].party = Democrat;
   // Reagan/Bush Sr.
   PresidentialTerms[2].date = D'1981.01.20';
   PresidentialTerms[2].party = Republican;
   // Clinton
   PresidentialTerms[3].date = D'1993.01.20';
   PresidentialTerms[3].party = Democrat;
   // Bush Jr.
   PresidentialTerms[4].date = D'2001.01.20';
   PresidentialTerms[4].party = Republican;
   // Obama
   PresidentialTerms[5].date = D'2009.01.20';
   PresidentialTerms[5].party = Democrat;
   // Trump
   PresidentialTerms[5].date = D'2017.01.20';
   PresidentialTerms[5].party = Republican;
   
   // END
   PresidentialTerms[6].date = D'2021.01.20';
   PresidentialTerms[6].party = None;

   MqlRates rates[];
   int bars = Bars(Symbol(), Period());
   bool PositionOpen = false;
   double Open = 0;
   double SumR = 0;
   double SumD = 0;
   
   if (CopyRates(Symbol(), Period(), 0, bars, rates) != bars)
   {
      Print("Failed to copy rates.");
      return;
   }
   Print("Rates copied.");
   
   int j = 0;
   for (int i = 0; i < bars; i++)
   {
      MqlDateTime dt;
      TimeToStruct(rates[i].time, dt);

      if (rates[i].time >= PresidentialTerms[j].date)
      {
         // Not the first time opening the position.
         if (Open != 0)
         {
            if (PresidentialTerms[j - 1].party == Democrat) SumD += rates[i].open - Open;
            else if (PresidentialTerms[j - 1].party == Republican) SumR += rates[i].open - Open;               
         }
         Open = rates[i].open;
         if (PresidentialTerms[j].party == None) break; // End reached.
         j++;
      }
   }
   // Correction for the current term.
   //SumR += rates[bars - 1].open - Open;
   
   Print("Republican sum: $", DoubleToString(SumR / Point(), 2));
   Print("Democrat sum: $", DoubleToString(SumD / Point(), 2));
}
//+------------------------------------------------------------------+